import click
import inflection
import os


def default(txt):
    return click.style(txt, fg='white', bold=True)


def prompt(txt):
    return click.style(txt, fg='green')


@click.command()
@click.argument('name')
@click.option(
    '--description',
    prompt=prompt('Description'),
    default=default('N/A')
)
@click.option(
    '--author',
    prompt=prompt('Author name'),
    default=lambda: default(os.environ.get('USER', ''))
)
@click.option(
    '--email',
    prompt=prompt('Author email'),
    default=lambda: default(os.environ.get('USER', '') + '@me.com')
)
@click.option('--version', prompt=prompt('Version'), default=default('0.0.1'))
@click.option(
    '--django-version', prompt=prompt('Django version'),
    default=default('1.10'))
def get_context(name, description, author, email, version, django_version):
    name = click.unstyle(name)
    description = click.unstyle(description)
    email = click.unstyle(email)
    author = click.unstyle(author)
    version = click.unstyle(version)
    django_version = click.unstyle(django_version)
    return {
        'app': inflection.underscore(name),
        'description': description,
        'author': author,
        'email': email,
        'version': version,
        'django_version': django_version
    }



from __future__ import absolute_import
import click
from .run import run


@click.argument('args', nargs=-1, type=click.UNPROCESSED)
@click.command(
    context_settings={
        'ignore_unknown_options': True
    }
)
def test(args):
    """Run tests."""
    args = [
        'pytest', 'tests', '--ds=tests.settings'
    ] + list(args)
    run.main(args, standalone_mode=False)



from __future__ import absolute_import
import click
from .run import run


@click.command()
@click.argument('port', required=False)
def server(port):
    """Start the Django dev server."""
    args = ['manage.py', 'runserver']
    if port:
        args.append(port)
    run.main(args)



import click
import inflection


@click.command()
@click.argument('name')
def get_context(name):
    """Generate a model with given name.

    You will need to run "dj migrate" to create
    the migration file and apply it to your local
    development database.

    For example:

        dj generate model Foo

        dj migrate
    """
    return {
        'name': inflection.underscore(name),
        'class_name': inflection.camelize(name)
    }



import os
import click
import sys
import subprocess
import signal


def get(
    directory,
    filter=None,
    depth=0,
    include_files=False,
    include_dirs=False
):
    if isinstance(filter, basestring):
        flt = lambda x: x == filter
    if not callable(filter):
        # if filter is None/unsupported type, allow all
        flt = lambda x: True
    else:
        flt = filter

    for root, dirs, files in os.walk(directory):
        search = []
        if include_files:
            search.extend(files)
        if include_dirs:
            search.extend(dirs)

        for file in search:
            file = os.path.join(root, file)
            if flt(file):
                yield file

        depth -= 1
        if depth == 0:
            break


def get_files(directory, filter=None, depth=0):
    return get(directory, filter, depth, include_files=True)


def get_directories(directory, filter=None, depth=0):
    return get(directory, filter, depth, include_dirs=True)


def get_last_touched(file):
    return os.path.getmtime(file) if os.path.exists(file) else None


def touch(file):
    with open(file, 'a'):
        os.utime(file, None)


def make_terminate_handler(process, signal=signal.SIGTERM):
    def inner(*args):
        try:
            os.killpg(os.getpgid(process.pid), signal)
        except OSError:
            pass
    return inner


def execute(
    command,
    abort=True,
    capture=False,
    verbose=False,
    echo=False,
    stream=None,
):
    """Run a command locally.

    Arguments:
        command: a command to execute.
        abort: If True, a non-zero return code will trigger an exception.
        capture: If True, returns the output of the command.
            If False, returns a subprocess result.
        echo: if True, prints the command before executing it.
        verbose: If True, prints the output of the command.
        stream: If set, stdout/stderr will be redirected to the given stream.
            Ignored if `capture` is True.
    """
    stream = stream or sys.stdout

    if echo:
        out = stream
        out.write(u'$ %s' % command)

    # Capture stdout and stderr in the same stream
    command = u'%s 2>&1' % command

    if verbose:
        out = stream
        err = stream
    else:
        out = subprocess.PIPE
        err = subprocess.PIPE

    process = subprocess.Popen(
        command,
        shell=True,
        stdout=out,
        stderr=err,
    )
    # propagate SIGTERM to all child processes within
    # the process group. this prevents subprocesses from
    # being orphaned when the current process is terminated
    signal.signal(
        signal.SIGTERM,
        make_terminate_handler(process)
    )

    # Wait for the process to complete
    stdout, _ = process.communicate()
    stdout = stdout.strip() if stdout else ''
    if not isinstance(stdout, unicode):
        stdout = stdout.decode('utf-8')

    if abort and process.returncode != 0:
        message = (
            u'Error #%d running "%s"%s' % (
                process.returncode,
                command,
                ':\n====================\n'
                '%s\n'
                '====================\n' % (
                    stdout
                ) if stdout else ''
            )
        )
        raise Exception(message)
    if capture:
        return stdout
    else:
        return process


def exists(program):
    try:
        execute("command -v %s" % program, verbose=False)
        return True
    except:
        return False


class StyleStdout(object):

    def __init__(self, **kwargs):
        self.kwargs = kwargs

    def write(self, message, **kwargs):
        copy = self.kwargs.copy()
        copy.update(kwargs)
        click.echo(click.style(message, **copy))

stdout = StyleStdout(fg='blue')



from __future__ import absolute_import
import click
from click.exceptions import ClickException
from dj.application import get_current_application
from dj.utils import style
from .base import stdout


@click.option('--quiet', is_flag=True, default=False)
@click.argument('args', nargs=-1, type=click.UNPROCESSED)
@click.command(
    context_settings={
        'ignore_unknown_options': True
    }
)
def run(quiet, args):
    """Run a local command.

    Examples:

    $ django run manage.py runserver

    ...
    """
    if not args:
        raise ClickException('pass a command to run')

    cmd = ' '.join(args)
    application = get_current_application()
    name = application.name
    return application.run(
        cmd,
        verbose=not quiet,
        abort=False,
        capture=True,
        env={
            'DJANGO_SETTINGS_MODULE': '%s.settings' % name
        }
    )



from __future__ import absolute_import
import click

from .base import MultiCommand
from .shell import shell
from .run import run
from .init import init
from .generate import generate
from .add import add
from .remove import remove
from .migrate import migrate
from .server import server
from .test import test
from .info import info
from .help import help


class _MultiCommand(MultiCommand):
    commands = {
        'migrate': migrate,
        'server': server,
        'shell': shell,
        'test': test,
        'run': run,
        'init': init,
        'generate': generate,
        'add': add,
        'remove': remove,
        'info': info,
        'help': help
    }


@click.command(cls=_MultiCommand)
def dj(*args, **kwargs):
    """DJ, the Django CLI."""
    pass


def execute(command):
    command = command.split(' ')
    return dj.main(command, standalone_mode=False)



import os
from .blueprint import Blueprint


class Addon(object):

    def __init__(self, name, directory):
        self.name = name
        self.module_name = name.replace('-', '_')
        self.directory = directory
        self.blueprints_directory = os.path.join(directory, 'blueprints')

    @property
    def blueprints(self):
        if not hasattr(self, '_blueprints'):
            self._blueprints = {
                b.name: b for b in self.get_blueprints()
            }
        return self._blueprints

    def get_blueprints(self):
        return Blueprint.get_blueprints(
            self.blueprints_directory,
            addon=self
        )

    def __repr__(self):
        return '%s (%s)' % (self.name, self.directory)



import os
import sys
from dj.utils.system import get_directories, get_files
from dj.utils.imports import load_module


class Blueprint(object):

    @classmethod
    def is_valid(cls, directory):
        # valid if there is a `templates` directory
        # and a `context.py` file
        templates = bool(
            list(get_directories(
                directory,
                filter=lambda x: x.endswith('/templates'),
                depth=1
            ))
        )
        context = bool(
            list(get_files(
                directory,
                filter=lambda x: x.endswith('/context.py'),
                depth=1
            ))
        )
        return templates and context

    @classmethod
    def get_blueprints(cls, directory, addon=None):
        return [
            Blueprint(d, addon=addon)
            for d in
            get_directories(
                directory,
                filter=lambda x: cls.is_valid(x),
                depth=1
            )
        ]

    def __init__(self, directory, addon=None):
        self.directory = directory
        self.addon = addon
        self.templates_directory = os.path.join(directory, 'templates')
        self.name = os.path.basename(self.directory)
        self.context = os.path.join(directory, 'context.py')

    @property
    def full_name(self):
        return (
            '%s.%s' % (self.addon.name, self.name)
            if self.addon else self.name
        )

    def load_context(self):
        return load_module(self.context).get_context

    def __repr__(self):
        return '%s (%s)' % (self.name, self.directory)


def get_core_blueprints():
    if getattr(sys, 'frozen', False):
        # get blueprints relative to sys.executable
        path = os.path.join(
            os.path.dirname(sys.executable),
            'dj/blueprints'
        )
    else:
        # get blueprints folder relative to this file
        path = os.path.abspath(
            os.path.join(
                os.path.dirname(__file__),
                'blueprints'
            )
        )

    blueprints = Blueprint.get_blueprints(path)
    return blueprints



from __future__ import absolute_import
import click
from dj.application import get_current_application


@click.argument('addon')
@click.option('--dev', is_flag=True)
@click.option(
    '--interactive/--not-interactive',
    is_flag=True,
    default=True
)
@click.command()
def add(addon, dev, interactive):
    """Add a dependency.

    Examples:

    $ django add dynamic-rest==1.5.0

    + dynamic-rest == 1.5.0
    """
    application = get_current_application()
    application.add(
        addon,
        dev=dev,
        interactive=interactive
    )



from unittest import TestCase
import click
from dj.test import TemporaryApplication
import time
import requests


class CLITestCase(TestCase):

    def test_create_new_app(self):
        print '* Generating application'
        application = TemporaryApplication()

        print '* Testing build/run'
        result = application.execute('run manage.py help --quiet')
        self.assertTrue('for help on a specific subcommand' in result, result)

        print '* Testing migration flow'
        result = application.execute('run manage.py migrate --quiet')
        self.assertTrue('Applying auth.0001_initial' in result, result)

        result = application.execute('run manage.py migrate --quiet')
        self.assertTrue('No migrations to apply' in result, result)

        print '* Testing model generation'
        application.execute('generate model foo --not-interactive')

        try:
            result = application.execute('test --ds=tests.settings')
        except Exception as e:
            e = click.unstyle(str(e))
            self.assertTrue(
                'no such table: %s_%s' % ('dummy', 'foo') in e,
                e
            )

        print '* Testing new migration flow'
        application.execute('run manage.py makemigrations --quiet')

        application.execute('test --ds=tests.settings')

        result = application.execute('run manage.py migrate --quiet')
        self.assertTrue('Applying dummy.0001_initial' in result, result)

        result = application.execute('run manage.py migrate --quiet')
        self.assertFalse('Applying dummy.0001_initial' in result, result)

        print '* Testing command generation'
        application.execute('generate command foobar')

        result = application.execute('run manage.py help --quiet')
        self.assertTrue('foobar' in result, result)

        result = application.execute('run manage.py foobar --quiet')
        self.assertTrue('foobar called' in result, result)

        result = click.unstyle(application.execute('info'))
        self.assertTrue('Django == 1.10' in result, result)

        print '* Testing server'
        server = application.execute('serve 9123', async=True)
        time.sleep(2)

        response = requests.get('http://localhost:9123')
        content = response.content
        self.assertTrue('It worked' in content, content)

        server.terminate()
